# The Kitaev chain model

Here's a simple example of the Kitaev chain Hamiltonian:

```julia
julia> using TopologicalNumbers
julia> function H₀(k, p)
            μ, Δ = p
            t = 1

            [
                -μ-2t*cos(k) 2im*Δ*sin(k)
                -2im*Δ*sin(k) μ+2t*cos(k)
            ]
        end
```

The band structure is computed as follows:

```julia
julia> H(k) = H₀(k, (-1.0, 0.5))
julia> showBand(H; value=false, disp=true)
```

![Band structure of Kitaev chain model](https://github.com/KskAdch/TopologicalNumbers.jl/assets/139373570/f3291b68-4326-4c12-b39c-73eeed924c66)

Next, we can calculate the winding numbers using `calcBerryPhase`:

```julia
julia> calcBerryPhase(H)
```

The output is:

```julia
(TopologicalNumber = [1, 1], Total = 0)
```

The first argument `TopologicalNumber` in the named tuple is an vector that stores the winding number for each band. 
The vector is arranged in order of bands, starting from the one with the lowest energy.
The second argument `Total` stores the total of the winding numbers for each band (mod 2).
`Total` is a quantity that should always return zero.



One-dimensional phase diagram is given by:

```julia
julia> H(k, p) = H₀(k, (p, 1.0))

julia> param = range(-3.0, 0, length=601)
julia> calcPhaseDiagram(H, param, "BerryPhase"; plot=true)
```

![One-dimensional phase diagram of Kitaev chain model](https://github.com/KskAdch/TopologicalNumbers.jl/assets/139373570/70d53e99-bc25-495e-a485-d267588296e3)

Also, two-dimensional phase diagram is given by:

```julia
julia> param1 = range(-3.0, 3.0, length=101)
julia> param2 = range(-1.0, 1.0, length=101)
julia> calcPhaseDiagram(H₀, param1, param2, "BerryPhase"; plot=true)
```

![Two-dimensional phase diagram of Kitaev chain model](https://github.com/KskAdch/TopologicalNumbers.jl/assets/139373570/5c41f96b-b622-4304-b8a3-3873b25d0a6a)